@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: ContextLoader.java,v 1.25 2004/08/08 06:36:56 gregwilkins Exp $
+// $Id: ContextLoader.java,v 1.36 2005/11/19 09:04:53 gregwilkins Exp $
 // Copyright 1999-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -28,9 +28,8 @@ import java.util.Arrays;
 import java.util.StringTokenizer;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 import org.mortbay.util.IO;
-import org.mortbay.util.LogSupport;
 import org.mortbay.util.Resource;
 
 /* ------------------------------------------------------------ */
@@ -43,7 +42,7 @@ import org.mortbay.util.Resource;
  * parent loader.  Java2 compliant loading, where the parent loader
  * always has priority, can be selected with the setJava2Complient method.
  *
- * @version $Id: ContextLoader.java,v 1.25 2004/08/08 06:36:56 gregwilkins Exp $
+ * @version $Id: ContextLoader.java,v 1.36 2005/11/19 09:04:53 gregwilkins Exp $
  * @author Greg Wilkins (gregw)
  */
 public class ContextLoader extends URLClassLoader
@@ -54,8 +53,8 @@ public class ContextLoader extends URLClassLoader
     private ClassLoader _parent;
     private PermissionCollection _permissions;
     private String _urlClassPath;
-    private boolean _fileClassPathWarning= false;
-
+    private HttpContext _context;
+    
     /* ------------------------------------------------------------ */
     /** Constructor.
      * @param classPath Comma separated path of filenames or URLs
@@ -67,8 +66,10 @@ public class ContextLoader extends URLClassLoader
         throws MalformedURLException, IOException
     {
         super(new URL[0], parent);
+        _context=context;
         _permissions= permisions;
         _parent= parent;
+        
         if (_parent == null)
             _parent= getSystemClassLoader();
 
@@ -97,8 +98,6 @@ public class ContextLoader extends URLClassLoader
                 }
                 else
                 {
-                    _fileClassPathWarning= true;
-
                     // Add resource or expand jar/
                     if (!resource.isDirectory() && file == null)
                     {
@@ -114,9 +113,17 @@ public class ContextLoader extends URLClassLoader
                         jar.deleteOnExit();
                         if (log.isDebugEnabled())
                             log.debug("Extract " + resource + " to " + jar);
-                        FileOutputStream out= new FileOutputStream(jar);
-                        IO.copy(in, out);
-                        out.close();
+                        FileOutputStream out = null;
+                        try
+                        {
+                            out= new FileOutputStream(jar);
+                            IO.copy(in, out);
+                        }
+                        finally
+                        {
+                            IO.close(out);
+                        }
+
                         URL url= jar.toURL();
                         addURL(url);
                         _urlClassPath=
@@ -162,9 +169,7 @@ public class ContextLoader extends URLClassLoader
     /* ------------------------------------------------------------ */
     public String toString()
     {
-        if (log.isDebugEnabled())
-            return "ContextLoader@" + hashCode() + "(" + _urlClassPath + ") / " + _parent.toString();
-        return "ContextLoader@" + hashCode();
+        return "ContextLoader@" + hashCode() + "(" + _urlClassPath + ")\n  --parent--> " + _parent.toString();
     }
 
     /* ------------------------------------------------------------ */
@@ -177,18 +182,18 @@ public class ContextLoader extends URLClassLoader
     }
 
     /* ------------------------------------------------------------ */
-    public synchronized Class loadClass(String name) throws ClassNotFoundException
+    public Class loadClass(String name) throws ClassNotFoundException
     {
         return loadClass(name, false);
     }
 
     /* ------------------------------------------------------------ */
-    protected synchronized Class loadClass(String name, boolean resolve) throws ClassNotFoundException
+    protected Class loadClass(String name, boolean resolve) throws ClassNotFoundException
     {
         Class c= findLoadedClass(name);
         ClassNotFoundException ex= null;
         boolean tried_parent= false;
-        if (c == null && (_java2compliant || isSystemPath(name)) && !isHiddenPath(name))
+        if (c == null && (_java2compliant || isSystemPath(name)) && !isServerPath(name) && _parent!=null)
         {
             if (log.isTraceEnabled())
                 log.trace("try loadClass " + name + " from " + _parent);
@@ -197,7 +202,7 @@ public class ContextLoader extends URLClassLoader
             {
                 c= _parent.loadClass(name);
                 if (log.isTraceEnabled())
-                    log.trace("loaded " + c);
+                    log.trace("p0 loaded " + c);
             }
             catch (ClassNotFoundException e)
             {
@@ -213,7 +218,7 @@ public class ContextLoader extends URLClassLoader
             {
                 c= this.findClass(name);
                 if (log.isTraceEnabled())
-                    log.trace("loaded " + c);
+                    log.trace("cx loaded " + c);
             }
             catch (ClassNotFoundException e)
             {
@@ -221,15 +226,15 @@ public class ContextLoader extends URLClassLoader
             }
         }
 
-        if (c == null && !tried_parent && !isHiddenPath(name))
+        if (c == null && !tried_parent && !isServerPath(name) && _parent!=null)
         {
             if (log.isTraceEnabled())
                 log.trace("try loadClass " + name + " from " + _parent);
             c= _parent.loadClass(name);
             if (log.isTraceEnabled())
-                log.trace("loaded " + c);
+                log.trace("p1 loaded " + c);
         }
-
+        
         if (c == null)
             throw ex;
 
@@ -240,7 +245,7 @@ public class ContextLoader extends URLClassLoader
     }
 
     /* ------------------------------------------------------------ */
-    public synchronized URL getResource(String name)
+    public URL getResource(String name)
     {
         URL url= null;
         boolean tried_parent= false;
@@ -281,37 +286,114 @@ public class ContextLoader extends URLClassLoader
     }
 
     /* ------------------------------------------------------------ */
-    public boolean isHiddenPath(String name)
+    public boolean isServerPath(String name)
     {
-        // Arbitrary list that covers the worst security problems.
-        // If you are worried by this, then use a permissions file!
-        return name.equals("org.mortbay.jetty.Server")
-            || name.equals("org.mortbay.http.HttpServer")
-            || name.startsWith("org.mortbay.start.")
-            || name.startsWith("org.mortbay.stop.");
+        name=name.replace('/','.');
+        while(name.startsWith("."))
+            name=name.substring(1);
+
+        String[] server_classes=_context.getServerClasses();
+        
+        if (server_classes!=null)
+        {
+            for (int i=0;i<server_classes.length;i++)
+            {
+                boolean result=true;
+                String c=server_classes[i];
+                if (c.startsWith("-"))
+                {
+                    c=c.substring(1);
+                    result=false;
+                }
+                
+                if (c.endsWith("."))
+                {
+                    if (name.startsWith(c))
+                        return result;
+                }
+                else if (name.equals(c))
+                {
+                    return result;
+                }
+            }
+        }
+        return false;
     }
 
     /* ------------------------------------------------------------ */
     public boolean isSystemPath(String name)
     {
-        return (
-            name.startsWith("java.")
-                || name.startsWith("javax.servlet.")
-                || name.startsWith("javax.xml.")
-                || name.startsWith("org.mortbay.")
-                || name.startsWith("org.xml.")
-                || name.startsWith("org.w3c.")
-                || name.startsWith("java/")
-                || name.startsWith("javax/servlet/")
-                || name.startsWith("javax/xml/")
-                || name.startsWith("org/mortbay/")
-                || name.startsWith("org/xml/")
-                || name.startsWith("org/w3c/")
-                || name.startsWith("/java/")
-                || name.startsWith("/javax/servlet/")
-                || name.startsWith("/javax/xml/")
-                || name.startsWith("/org/mortbay/")
-                || name.startsWith("/org/xml/")
-                || name.startsWith("/org/w3c/"));
+        name=name.replace('/','.');
+        while(name.startsWith("."))
+            name=name.substring(1);
+
+        String[] system_classes=_context.getSystemClasses();
+        if (system_classes!=null)
+        {
+            for (int i=0;i<system_classes.length;i++)
+            {
+                boolean result=true;
+                String c=system_classes[i];
+                if (c.startsWith("-"))
+                {
+                    c=c.substring(1);
+                    result=false;
+                }
+                
+                if (c.endsWith("."))
+                {
+                    if (name.startsWith(c))
+                        return result;
+                }
+                else if (name.equals(c))
+                    return result;
+            }
+        }
+        
+        return false;
+    }
+
+    /* ------------------------------------------------------------ */
+    public void destroy()
+    {
+        this._parent=null;
+        this._permissions=null;
+        this._urlClassPath=null;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @return Returns the serverClasses.
+     */
+    String[] getServerClasses()
+    {
+        return _context.getServerClasses();
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @param serverClasses The serverClasses to set.
+     */
+    void setServerClasses(String[] serverClasses)
+    {
+        _context.setServerClasses(serverClasses);
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @return Returns the systemClasses.
+     */
+    String[] getSystemClasses()
+    {
+        return _context.getSystemClasses();
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @param systemClasses The systemClasses to set.
+     */
+    void setSystemClasses(String[] systemClasses)
+    {
+        _context.setSystemClasses(systemClasses);
     }
 }
