@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: HttpConnection.java,v 1.79 2004/08/28 00:39:29 gregwilkins Exp $
+// $Id: HttpConnection.java,v 1.92 2005/12/06 00:51:40 gregwilkins Exp $
 // Copyright 199-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,7 +24,7 @@ import java.util.Enumeration;
 import java.util.List;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 import org.mortbay.util.InetAddrPort;
 import org.mortbay.util.LineInput;
 import org.mortbay.util.LogSupport;
@@ -45,7 +45,7 @@ import org.mortbay.util.StringUtil;
  *
  * @see HttpListener
  * @see HttpServer
- * @version $Id: HttpConnection.java,v 1.79 2004/08/28 00:39:29 gregwilkins Exp $
+ * @version $Id: HttpConnection.java,v 1.92 2005/12/06 00:51:40 gregwilkins Exp $
  * @author Greg Wilkins (gregw)
  */
 public class HttpConnection
@@ -68,13 +68,14 @@ public class HttpConnection
     protected HttpResponse _response;
     protected boolean _persistent;
     protected boolean _keepAlive;
+    protected int _dotVersion;
 
     private HttpListener _listener;
     private HttpInputStream _inputStream;
     private HttpOutputStream _outputStream;
     private boolean _close;
-    private int _dotVersion;
     private boolean _firstWrite;
+    private boolean _completing;
     private Thread _handlingThread;
     
     private InetAddress _remoteInetAddress;
@@ -134,9 +135,7 @@ public class HttpConnection
         _request = new HttpRequest(this); 
         _response = new HttpResponse(this);
 
-        _resolveRemoteHost =
-            _listener!=null &&
-            _listener.getHttpServer().getResolveRemoteHost();
+        _resolveRemoteHost=_listener!=null && _listener.getHttpServer()!=null && _listener.getHttpServer().getResolveRemoteHost();
     }
 
     /* ------------------------------------------------------------ */
@@ -264,6 +263,7 @@ public class HttpConnection
         throws IOException
     {
         try{
+            _completing=true;
             _outputStream.close();
             _inputStream.close();
         }
@@ -461,14 +461,13 @@ public class HttpConnection
         // check and enable requests transfer encodings.
         String transfer_coding=
             _request.getField(HttpFields.__TransferEncoding);
-        
+
         if (transfer_coding!=null && transfer_coding.length()>0)
         {
             // Handling of codings other than chunking is now
             // the responsibility of handlers, filters or servlets.
             // Thanks to the compression filter, we now don't know if
             // what we can handle here.
-
             if (transfer_coding.equalsIgnoreCase(HttpFields.__Chunked) ||
                 StringUtil.endsWithIgnoreCase(transfer_coding,HttpFields.__Chunked))
                 _inputStream.setChunking();
@@ -502,16 +501,9 @@ public class HttpConnection
         String expect=_request.getField(HttpFields.__Expect);
         if (expect!=null && expect.length()>0)
         {
-            if (StringUtil.asciiToLowerCase(expect)
-                .equals(HttpFields.__ExpectContinue))
+            if (StringUtil.asciiToLowerCase(expect).equals(HttpFields.__ExpectContinue))
             {
-                // Send continue if no body available yet.
-                if (_inputStream.available()<=0)
-                {
-                    OutputStream real_out=_outputStream.getOutputStream();
-                    real_out.write(HttpResponse.__Continue);
-                    real_out.flush();
-                }
+                _inputStream.setExpectContinues(_outputStream.getOutputStream());
             }
             else
                 throw new HttpException(HttpResponse.__417_Expectation_Failed);
@@ -530,7 +522,6 @@ public class HttpConnection
         // Persistent unless requested otherwise
         _persistent=!_close;
     }
-    
 
     /* ------------------------------------------------------------ */
     /** Output Notifications.
@@ -559,7 +550,7 @@ public class HttpConnection
               break;
               
           case OutputObserver.__RESET_BUFFER:
-              _firstWrite=false;
+              resetBuffer();
               break;
               
           case OutputObserver.__COMMITING:
@@ -567,10 +558,14 @@ public class HttpConnection
               break;
               
           case OutputObserver.__CLOSING:
-              if (_response!=null &&
-                  !_response.isCommitted() &&
+              
+              if (_response!=null)
+              {
+                  completing();
+                  if (!_response.isCommitted() &&
                   _request.getState()==HttpMessage.__MSG_RECEIVED)
                   commit();
+              }
               break;
               
           case OutputObserver.__CLOSED:
@@ -589,89 +584,28 @@ public class HttpConnection
         if (_response.isCommitted())
             return;
         
-        // Determine how to limit content length and
-        // enable output transfer encodings
-
-        String transfer_coding=_response.getField(HttpFields.__TransferEncoding);
-        if (transfer_coding==null ||
-            transfer_coding.length()==0 ||
-            HttpFields.__Identity.equalsIgnoreCase(transfer_coding))
-        {
-            switch(_dotVersion)
-            {
-              case 1:
-                  {
-                      // if (not closed and no length)
-                      if ((!HttpFields.__Close.equals(_response.getField(HttpFields.__Connection)))&&
-                          (_response.getField(HttpFields.__ContentLength)==null))
-                      {
-                          // Chunk it!
-                          _response.setField(HttpFields.__TransferEncoding,HttpFields.__Chunked);
-                          _outputStream.setChunking();
-                      }
-                      break;
-                  }
-              case 0:
-                  {
-                      // If we dont have a content length (except 304 replies), 
-                  	  // or we have been requested to close
-		              // then we can't be persistent 
-                      if (!_keepAlive || !_persistent ||
-                          HttpResponse.__304_Not_Modified!=_response.getStatus() &&
-                          _response.getField(HttpFields.__ContentLength)==null ||
-                          HttpFields.__Close.equals(_response.getField(HttpFields.__Connection)))
-                      {
-                          _persistent=false;
-                          _response.setField(HttpFields.__Connection,HttpFields.__Close);
-                          _keepAlive=false;
-                      }
-                      else if (_keepAlive)
-                          _response.setField(HttpFields.__Connection,
-                                             HttpFields.__KeepAlive);
-                      break;
-                  }
-              default:
-                  _keepAlive=false;
-                  _persistent=false;
-            }
-        }
-        else if (_dotVersion<1)
-        {
-            // Error for transfer encoding to be set in HTTP/1.0
-            _response.removeField(HttpFields.__TransferEncoding);
-            throw new HttpException(HttpResponse.__501_Not_Implemented,
-                                    "Transfer-Encoding not supported in HTTP/1.0");
-        }
-        else
-        {
-            // Use transfer encodings to determine length
-            _response.removeField(HttpFields.__ContentLength);
-            _outputStream.setChunking();
-
-            if (!HttpFields.__Chunked.equalsIgnoreCase(transfer_coding))
-            {
-                // Check against any TE field
-                List te = _request.getAcceptableTransferCodings();
-                Enumeration enm =
-                    _response.getFieldValues(HttpFields.__TransferEncoding,
-                                             HttpFields.__separators);
-                while (enm.hasMoreElements())
-                {
-                    String coding=(String)enm.nextElement();
-                    if (HttpFields.__Identity.equalsIgnoreCase(coding) ||
-                        HttpFields.__Chunked.equalsIgnoreCase(coding))
-                        continue;
-                    if (te==null || !te.contains(coding))
-                        throw new HttpException(HttpResponse.__501_Not_Implemented,coding);
-                }
-            }
-        }
-
         // Nobble the OutputStream for HEAD requests
         if (HttpRequest.__HEAD.equals(_request.getMethod()))
-            _outputStream.nullOutput();
+            _outputStream.nullOutput();    
+            
+        int length=_response.getIntField(HttpFields.__ContentLength);
+        if (length>=0)
+            _outputStream.setContentLength(length);
     }
 
+    /**
+     * Reset headers after response reset
+     */
+    private void resetBuffer()
+    {
+    }
+
+    /* ------------------------------------------------------------ */
+    /* Signal that the next commit/flush is the last */
+    void completing()
+    {
+        _completing=true;
+    }
     
     /* ------------------------------------------------------------ */
     protected void commit()
@@ -680,80 +614,132 @@ public class HttpConnection
         if (_response.isCommitted())
             return;
 
-        // Mark request as handled.
-        _request.setHandled(true);
+        int status=_response.getStatus();
+        int length=-1;
+        
+        // Check if there is missing content expectations
+        if (_inputStream.getExpectContinues()!=null)
+        {
+            // No input read yet - so assume it never will be
+            _inputStream.setExpectContinues(null);
+            _inputStream.unsafeSetContentLength(0);
+        }
         
         // Handler forced close, listener stopped or no idle threads left.
-        _close=HttpFields.__Close.equals(_response.getField(HttpFields.__Connection));
-        if (!_close && _listener!=null && (!_listener.isStarted()||_listener.isOutOfResources()))
+        boolean has_close=HttpFields.__Close.equals(_response.getField(HttpFields.__Connection));
+        if (!_persistent || _close || _listener!=null && (!_listener.isStarted()||_listener.isOutOfResources()))
         {
             _close=true;
-            _response.setField(HttpFields.__Connection,
-                               HttpFields.__Close);
+            if (!has_close)
+                _response.setField(HttpFields.__Connection,HttpFields.__Close);
+            has_close=true;
         }
         if (_close)
             _persistent=false;
-
         
-        // if we have no content or encoding, and no content length
-        int status = _response.getStatus();
-        if (!_outputStream.isWritten() &&
-            !_response.containsField(HttpFields.__ContentLength) &&
-            !_response.containsField(HttpFields.__TransferEncoding))
+        // Determine how to limit content length
+        if (_persistent)
         {
-            // Special case for responses with no content.
-            if (status==HttpResponse.__304_Not_Modified ||
-                status==HttpResponse.__204_No_Content)
+            switch(_dotVersion)
             {
-                if (_persistent && _keepAlive && _dotVersion==0)
-                    _response.setField(HttpFields.__Connection,
-                                       HttpFields.__KeepAlive);
-            }
-            else
-            {
-                if(_persistent)
+                case 1:
                 {
-                    switch (_dotVersion)
+                    String transfer_coding=_response.getField(HttpFields.__TransferEncoding);
+                    if (transfer_coding==null || transfer_coding.length()==0 || HttpFields.__Identity.equalsIgnoreCase(transfer_coding))
                     {
-                    case 0:
+                        // if (can have content and no content length)
+                        if (status!=HttpResponse.__304_Not_Modified &&
+                            status!=HttpResponse.__204_No_Content &&
+                            _response.getField(HttpFields.__ContentLength)==null)
                         {
-                            _close=true;
-                            _persistent=false;
-                            _response.setField(HttpFields.__Connection,
-                                               HttpFields.__Close);
+                            if (_completing)
+                            {
+                                length=_outputStream.getBytesWritten();
+                                _response.setContentLength(length);
+                            }
+                            else
+                            {   
+                                // Chunk it!
+                                _response.setField(HttpFields.__TransferEncoding,HttpFields.__Chunked);
+                                _outputStream.setChunking();
+                            }
                         }
-                        break;
-                    case 1:
+                    }
+                    else
+                    {
+                        // Use transfer encodings to determine length
+                        _response.removeField(HttpFields.__ContentLength);
+                        _outputStream.setChunking();
+
+                        if (!HttpFields.__Chunked.equalsIgnoreCase(transfer_coding))
                         {
-                            // Nobble the OutputStream for HEAD requests
-                            if (HttpRequest.__HEAD.equals(_request.getMethod()))
-                                _outputStream.nullOutput();
-
-                            // force chunking on.
-                            _response.setField(HttpFields.__TransferEncoding,
-                                               HttpFields.__Chunked);
-                            _outputStream.setChunking();
-
+                            // Check against any TE field
+                            List te = _request.getAcceptableTransferCodings();
+                            Enumeration enm =
+                                _response.getFieldValues(HttpFields.__TransferEncoding,
+                                                         HttpFields.__separators);
+                            while (enm.hasMoreElements())
+                            {
+                                String coding=(String)enm.nextElement();
+                                if (HttpFields.__Identity.equalsIgnoreCase(coding) ||
+                                    HttpFields.__Chunked.equalsIgnoreCase(coding))
+                                    continue;
+                                if (te==null || !te.contains(coding))
+                                    throw new HttpException(HttpResponse.__501_Not_Implemented,coding);
+                            }
                         }
-                        break;
-                        
-                    default:
-                        _close=true;
-                        _response.setField(HttpFields.__Connection,
-                                           HttpFields.__Close);
-                        break;
                     }
                 }
-                else
+                break;
+                
+                case 0:
+                {
+                    // if (can have content and no content length)
+                    _response.removeField(HttpFields.__TransferEncoding);
+                    if (_keepAlive)
+                    {
+                        if (status!=HttpResponse.__304_Not_Modified &&
+                            status!=HttpResponse.__204_No_Content &&
+                            _response.getField(HttpFields.__ContentLength)==null)
+                        {
+                            if (_completing)
+                            {
+                                length=_outputStream.getBytesWritten();
+                                _response.setContentLength(length);
+                                _response.setField(HttpFields.__Connection,HttpFields.__KeepAlive);
+                            }
+                            else
+                            {
+                                _response.setField(HttpFields.__Connection,HttpFields.__Close);
+                                has_close=_close=true;
+                                _persistent=false;
+                            }
+                        }
+                        else
+                            _response.setField(HttpFields.__Connection,HttpFields.__KeepAlive);
+                    }
+                    else if (!has_close)
+                        _response.setField(HttpFields.__Connection,HttpFields.__Close);
+                        
+                        
+                    break;
+                }
+                default:
                 {
                     _close=true;
-                    _response.setField(HttpFields.__Connection,
-                                       HttpFields.__Close);
+                    _persistent=false;
+                    _keepAlive=false;
                 }
             }
         }
-
+        
+        
+        // Mark request as handled.
+        _request.setHandled(true);
+        
         _outputStream.writeHeader(_response);
+        _outputStream.flush();
+        
     }
 
     
@@ -763,8 +749,9 @@ public class HttpConnection
      */
     private void exception(Throwable e)
     {
-	try{
-	    _persistent=false;
+        try
+        {
+            _persistent=false;
             int error_code=HttpResponse.__500_Internal_Server_Error;
             
             if (e instanceof HttpException)
@@ -786,21 +773,21 @@ public class HttpConnection
             {
                 _request.setAttribute("javax.servlet.error.exception_type",e.getClass());
                 _request.setAttribute("javax.servlet.error.exception",e);
-
+                
                 if (_request==null)
                     log.warn(LogSupport.EXCEPTION,e);
                 else
                     log.warn(_request.getRequestLine(),e);
             }
             
-	    if (_response != null && !_response.isCommitted())
-	    {
-		_response.reset();
-		_response.removeField(HttpFields.__TransferEncoding);
-		_response.setField(HttpFields.__Connection,HttpFields.__Close);
-		_response.sendError(error_code);
-	    }
-	}
+            if (_response != null && !_response.isCommitted())
+            {
+                _response.reset();
+                _response.removeField(HttpFields.__TransferEncoding);
+                _response.setField(HttpFields.__Connection,HttpFields.__Close);
+                _response.sendError(error_code);
+            }
+        }
         catch(Exception ex)
         {
             LogSupport.ignore(log,ex);
@@ -872,7 +859,6 @@ public class HttpConnection
     protected void readRequest()
         throws IOException
     {
-        log.debug("readRequest() ...");
         _request.readHeader((LineInput)(_inputStream)
                             .getInputStream());
     }
@@ -915,6 +901,7 @@ public class HttpConnection
             _close=false;
             _keepAlive=false;
             _firstWrite=false;
+            _completing=false;
             _dotVersion=0;
 
             // Read requests
@@ -947,11 +934,8 @@ public class HttpConnection
             // Common fields on the response
             _response.setVersion(HttpMessage.__HTTP_1_1);
             _response.setField(HttpFields.__Date,_request.getTimeStampStr());
-            if (!Version.__paranoid)
-            {
-                _response.setField(HttpFields.__Server,Version.__VersionDetail);
-                // _response.setField(HttpFields.__ServletEngine,Version.__ServletEngine);
-            }
+            if (!Version.isParanoid())
+                _response.setField(HttpFields.__Server,Version.getDetail());
             
             // Handle Connection header field
             Enumeration connectionValues =
@@ -987,7 +971,7 @@ public class HttpConnection
             else if (_dotVersion!=-1)
                 throw new HttpException(HttpResponse.__505_HTTP_Version_Not_Supported);
 
-            if(log.isDebugEnabled())log.debug("REQUEST:\n"+_request);
+            if(log.isDebugEnabled())log.debug("REQUEST from "+_listener+":\n"+_request);
 
             // handle HttpListener handlers
             if (!_request.isHandled() && _listener.getHttpHandler()!=null)
@@ -1023,10 +1007,21 @@ public class HttpConnection
                 
             // Complete the request
             if (_persistent)
-            {                    
+            {         
+                boolean no_continue_sent=false;
                 try{
-                    // Read remaining input
-                    while(_inputStream.skip(4096)>0 || _inputStream.read()>=0);
+                    if (_inputStream.getExpectContinues()!=null)
+                    {
+                        _inputStream.setExpectContinues(null);
+                        no_continue_sent=true;
+                    }
+                    else 
+                    {
+                        int remaining = _inputStream.getContentLength();
+                        if (remaining!=0)
+                            // Read remaining input
+                            while(_inputStream.skip(4096)>0 || _inputStream.read()>=0);
+                    }
                 }
                 catch(IOException e)
                 {
@@ -1039,12 +1034,11 @@ public class HttpConnection
                 }
                     
                 // Check for no more content
-                if (_inputStream.getContentLength()>0)
+                if (!no_continue_sent && _inputStream.getContentLength()>0)
                 {
                     _inputStream.setContentLength(0);
                     _persistent=false;
-                    exception (new HttpException(HttpResponse.__400_Bad_Request,
-                                                 "Missing Content"));
+                    exception (new HttpException(HttpResponse.__400_Bad_Request,"Missing Content"));
                 }
                 
                 // Commit the response
@@ -1174,4 +1168,5 @@ public class HttpConnection
             _httpServer.statsCloseConnection(_tmpTime-_openTime,_requests);
         }
     }
+
 }
