@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: HttpResponse.java,v 1.52 2004/05/09 20:31:40 gregwilkins Exp $
+// $Id: HttpResponse.java,v 1.61 2005/10/26 08:10:14 gregwilkins Exp $
 // Copyright 199-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,7 +24,7 @@ import java.util.HashMap;
 import javax.servlet.http.Cookie;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 import org.mortbay.util.LogSupport;
 import org.mortbay.util.StringUtil;
 import org.mortbay.util.TypeUtil;
@@ -41,7 +41,7 @@ import org.mortbay.util.UrlEncoded;
  * synchronized if it is used by multiple threads.
  *
  * @see HttpRequest
- * @version $Id: HttpResponse.java,v 1.52 2004/05/09 20:31:40 gregwilkins Exp $
+ * @version $Id: HttpResponse.java,v 1.61 2005/10/26 08:10:14 gregwilkins Exp $
  * @author Greg Wilkins (gregw)
  */
 public class HttpResponse extends HttpMessage
@@ -178,7 +178,7 @@ public class HttpResponse extends HttpMessage
     /** Set the HttpContext handling this reponse.
      * @param context The HttpContext handling this reponse.
      */
-    void setHttpContext(HttpContext context)
+    public void setHttpContext(HttpContext context)
     {
         _httpContext=context;
     }
@@ -212,6 +212,10 @@ public class HttpResponse extends HttpMessage
             _status= __200_OK;
             _reason=null;
             super.reset();
+
+            setField(HttpFields.__Date,getRequest().getTimeStampStr());
+            if (!Version.isParanoid())
+                setField(HttpFields.__Server,Version.getDetail());
         }
         catch(Exception e)
         {
@@ -270,9 +274,9 @@ public class HttpResponse extends HttpMessage
             _state=__MSG_BAD;
             writer.write(_version);
             writer.write(' ');
-	    writer.write('0'+((_status/100)%10));
-	    writer.write('0'+((_status/10)%10));
-	    writer.write('0'+(_status%10));
+            writer.write('0'+((_status/100)%10));
+            writer.write('0'+((_status/10)%10));
+            writer.write('0'+(_status%10));
             writer.write(' ');
             writer.write(getReason());
             writer.write(HttpFields.__CRLF);
@@ -309,14 +313,12 @@ public class HttpResponse extends HttpMessage
     {
         _reason=reason;
     }
-      
     
-    /* ------------------------------------------------------------- */
-    /** Send Error Response.
-     */
-    public void sendError(int code,String message) 
-        throws IOException
-    {        
+
+    /* -------------------------------------------------------------- */
+    public void setStatus(int code,String message)
+    {
+        setStatus(code);
         Integer code_integer=TypeUtil.newInteger(code);
         if (message == null)
         {
@@ -327,10 +329,19 @@ public class HttpResponse extends HttpMessage
         }
         else
             setReason(UrlEncoded.encodeString(message));
-        HttpRequest request=getHttpRequest();
-             
+    }
+      
+    
+    /* ------------------------------------------------------------- */
+    /** Send Error Response.
+     */
+    public void sendError(int code,String message) 
+        throws IOException
+    {        
+        setStatus(code,message);
+        
         // Generate normal error page.
-        setStatus(code);
+        HttpRequest request=getHttpRequest();
         
         // If we are allowed to have a body 
         if (code!=__204_No_Content &&
@@ -395,7 +406,7 @@ public class HttpResponse extends HttpMessage
     public void addSetCookie(String name,
                              String value)
     {
-        addSetCookie(new Cookie(name,value),false);
+        _header.addSetCookie(new Cookie(name,value));
     }
     
     /* -------------------------------------------------------------- */
@@ -403,17 +414,14 @@ public class HttpResponse extends HttpMessage
      */
     public void addSetCookie(Cookie cookie)
     {
-        addSetCookie(cookie,false);
+        _header.addSetCookie(cookie);
     }
     
-    /* -------------------------------------------------------------- */
-    /** Add a Set-Cookie field.
-     * @param cookie The cookie.
-     * @param cookie2 If true, use the alternate cookie 2 header
-     */
-    public void addSetCookie(Cookie cookie, boolean cookie2)
+
+    /* ------------------------------------------------------------ */
+    public void completing()
     {
-        _header.addSetCookie(cookie,cookie2);
+        getHttpConnection().completing();
     }
     
     /* ------------------------------------------------------------ */
@@ -425,6 +433,7 @@ public class HttpResponse extends HttpMessage
     {
         if (!isCommitted())
             getOutputStream().flush();
+        getHttpConnection().commit();
     }
     
     /* ------------------------------------------------------------ */
