@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: ResourceHandler.java,v 1.59 2004/05/09 20:32:06 gregwilkins Exp $
+// $Id: ResourceHandler.java,v 1.66 2005/08/24 08:18:17 gregwilkins Exp $
 // Copyright 199-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,14 +22,14 @@ import java.util.Enumeration;
 import java.util.List;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.mortbay.http.HttpContext;
+import org.mortbay.log.LogFactory;
 import org.mortbay.http.HttpException;
 import org.mortbay.http.HttpFields;
 import org.mortbay.http.HttpRequest;
 import org.mortbay.http.HttpResponse;
 import org.mortbay.http.InclusiveByteRange;
 import org.mortbay.http.MultiPartResponse;
+import org.mortbay.http.ResourceCache;
 import org.mortbay.util.CachedResource;
 import org.mortbay.util.IO;
 import org.mortbay.util.LogSupport;
@@ -46,7 +46,7 @@ import org.mortbay.util.URI;
  * A simple memory cache is also provided to reduce file I/O.
  * HTTP/1.1 ranges are supported.
  * 
- * @version $Id: ResourceHandler.java,v 1.59 2004/05/09 20:32:06 gregwilkins Exp $
+ * @version $Id: ResourceHandler.java,v 1.66 2005/08/24 08:18:17 gregwilkins Exp $
  * @author Nuno Preguia
  * @author Greg Wilkins
  */
@@ -364,8 +364,8 @@ public class ResourceHandler extends AbstractHttpHandler
             // If we have meta data for the file
             // Try a direct match for most common requests. Avoids
             // parsing the date.
-            HttpContext.ResourceMetaData metaData =
-                (HttpContext.ResourceMetaData)resource.getAssociate();
+            ResourceCache.ResourceMetaData metaData =
+                (ResourceCache.ResourceMetaData)resource.getAssociate();
             if (metaData!=null)
             {
                 String ifms=request.getField(HttpFields.__IfModifiedSince);
@@ -573,10 +573,10 @@ public class ResourceHandler extends AbstractHttpHandler
     public void writeHeaders(HttpResponse response,Resource resource, long count)
         throws IOException
     {
-        HttpContext.ResourceMetaData metaData =
-            (HttpContext.ResourceMetaData)resource.getAssociate();
+        ResourceCache.ResourceMetaData metaData =
+            (ResourceCache.ResourceMetaData)resource.getAssociate();
 
-        response.setContentType(metaData.getEncoding());
+        response.setContentType(metaData.getMimeType());
         if (count != -1)
         {
             if (count==resource.length())
@@ -685,9 +685,9 @@ public class ResourceHandler extends AbstractHttpHandler
         //  216 response which does not require an overall 
         //  content-length header
         //
-        HttpContext.ResourceMetaData metaData =
-            (HttpContext.ResourceMetaData)resource.getAssociate();
-        String encoding = metaData.getEncoding();
+        ResourceCache.ResourceMetaData metaData =
+            (ResourceCache.ResourceMetaData)resource.getAssociate();
+        String encoding = metaData.getMimeType();
         MultiPartResponse multi = new MultiPartResponse(response);
         response.setStatus(HttpResponse.__206_Partial_Content);
         response.setReason((String)HttpResponse.__statusMsg
@@ -773,7 +773,7 @@ public class ResourceHandler extends AbstractHttpHandler
         if (data==null)
         {
             String base = URI.addPaths(request.getPath(),"/");
-            String dir = resource.getListHTML(base,parent);
+            String dir = resource.getListHTML(URI.encodePath(base),parent);
             if (dir==null)
             {
                 response.sendError(HttpResponse.__403_Forbidden,
