@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: AbstractSessionManager.java,v 1.37 2004/05/09 20:32:26 gregwilkins Exp $
+// $Id: AbstractSessionManager.java,v 1.51 2005/08/24 09:03:22 gregwilkins Exp $
 // Copyright 199-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,7 +17,6 @@ package org.mortbay.jetty.servlet;
 
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.ConcurrentModificationException;
 import java.util.Enumeration;
 import java.util.EventListener;
 import java.util.HashMap;
@@ -27,6 +26,7 @@ import java.util.Map;
 import java.util.Random;
 
 import javax.servlet.ServletContext;
+import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
 import javax.servlet.http.HttpSessionAttributeListener;
@@ -37,9 +37,11 @@ import javax.servlet.http.HttpSessionEvent;
 import javax.servlet.http.HttpSessionListener;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
+import org.mortbay.http.HttpOnlyCookie;
 import org.mortbay.util.LazyList;
 import org.mortbay.util.LogSupport;
+import org.mortbay.util.MultiMap;
 
 
 /* ------------------------------------------------------------ */
@@ -55,19 +57,20 @@ import org.mortbay.util.LogSupport;
  * org.mortbay.jetty.servlet.AbstractSessionManager.23Notifications is set to
  * true, the 2.3 servlet spec notification style will be used.
  * <p>
- * @version $Id: AbstractSessionManager.java,v 1.37 2004/05/09 20:32:26 gregwilkins Exp $
+ * @version $Id: AbstractSessionManager.java,v 1.51 2005/08/24 09:03:22 gregwilkins Exp $
  * @author Greg Wilkins (gregw)
  */
 public abstract class AbstractSessionManager implements SessionManager
 {
     private static Log log = LogFactory.getLog(AbstractSessionManager.class);
-
+    
     /* ------------------------------------------------------------ */
     public final static int __distantFuture = 60*60*24*7*52*20;
+    private final static String __NEW_SESSION_ID="org.mortbay.jetty.newSessionId";
 
     /* ------------------------------------------------------------ */
-    public final static boolean __24SessionDestroyed=
-        Boolean.getBoolean("org.mortbay.jetty.servlet.AbstractSessionManager.24SessionDestroyed");
+    /* global Map of ID to session */
+    protected static MultiMap __allSessions=new MultiMap();  
     
     /* ------------------------------------------------------------ */
     // Setting of max inactive interval for new sessions
@@ -75,7 +78,6 @@ public abstract class AbstractSessionManager implements SessionManager
     private int _dftMaxIdleSecs = -1;
     private int _scavengePeriodMs = 30000;
     private String _workerName ;
-    private boolean _useRequestedId=false;
     protected transient ArrayList _sessionListeners=new ArrayList();
     protected transient ArrayList _sessionAttributeListeners=new ArrayList();
     protected transient Map _sessions;
@@ -83,7 +85,11 @@ public abstract class AbstractSessionManager implements SessionManager
     protected transient ServletHandler _handler;
     protected int _minSessions = 0;
     protected int _maxSessions = 0;
-
+    protected boolean _crossContextSessionIDs=false;
+    protected boolean _secureCookies=false;
+    protected boolean _httpOnly=false;
+    protected boolean _invalidateGlobal=true;
+    
     private transient SessionScavenger _scavenger = null;
     
     /* ------------------------------------------------------------ */
@@ -97,25 +103,53 @@ public abstract class AbstractSessionManager implements SessionManager
     {
         _random=random;
     }
-
+    
+    
+    
     /* ------------------------------------------------------------ */
     /** 
      * @return True if requested session ID are first considered for new
+     * @deprecated use getCrossContextSessionIDs
      * session IDs
      */
     public boolean getUseRequestedId()
     {
-        return _useRequestedId;
+        return _crossContextSessionIDs;
     }
-
+    
     /* ------------------------------------------------------------ */
-    /** 
+    /** Set Use Requested ID.
      * @param useRequestedId True if requested session ID are first considered for new
+     * @deprecated use setCrossContextSessionIDs
      * session IDs
      */
     public void setUseRequestedId(boolean useRequestedId)
+    {   
+        _crossContextSessionIDs = useRequestedId;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /** 
+     * @return True if cross context session IDs are first considered for new
+     * session IDs
+     */
+    public boolean getCrossContextSessionIDs()
     {
-        _useRequestedId = useRequestedId;
+        return _crossContextSessionIDs;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /** Set Cross Context sessions IDs
+     * This option activates a mode where a requested session ID can be used to create a 
+     * new session. This facilitates the sharing of session cookies when cross context
+     * dispatches use sessions.   
+     * 
+     * @param useRequestedId True if cross context session ID are first considered for new
+     * session IDs
+     */
+    public void setCrossContextSessionIDs(boolean useRequestedId)
+    {   
+        _crossContextSessionIDs = useRequestedId;
     }
     
     /* ------------------------------------------------------------ */
@@ -129,32 +163,32 @@ public abstract class AbstractSessionManager implements SessionManager
     {
         return Collections.unmodifiableMap(_sessions);
     }
-
-   /* ------------------------------------------------------------ */
-   public int getSessions ()
-   {
-      return _sessions.size ();
-   }
-
-   /* ------------------------------------------------------------ */
-   public int getMinSessions ()
-   {
-      return _minSessions;
-   }
-
-   /* ------------------------------------------------------------ */
-   public int getMaxSessions ()
-   {
-      return _maxSessions;
-   }
-
-   /* ------------------------------------------------------------ */
-   public void resetStats ()
-   {
-      _minSessions =  _sessions.size ();
-      _maxSessions = _sessions.size ();
-   }
-
+    
+    /* ------------------------------------------------------------ */
+    public int getSessions ()
+    {
+        return _sessions.size ();
+    }
+    
+    /* ------------------------------------------------------------ */
+    public int getMinSessions ()
+    {
+        return _minSessions;
+    }
+    
+    /* ------------------------------------------------------------ */
+    public int getMaxSessions ()
+    {
+        return _maxSessions;
+    }
+    
+    /* ------------------------------------------------------------ */
+    public void resetStats ()
+    {
+        _minSessions =  _sessions.size ();
+        _maxSessions = _sessions.size ();
+    }
+    
     /* ------------------------------------------------------------ */
     /* new Session ID.
      * If the request has a requestedSessionID which is unique, that is used.
@@ -168,10 +202,24 @@ public abstract class AbstractSessionManager implements SessionManager
      */
     private String newSessionId(HttpServletRequest request,long created)
     {
-        synchronized(_sessions)
+        synchronized(__allSessions)
         {
-            String id=_useRequestedId?request.getRequestedSessionId():null;
-            while (id==null || id.length()==0 || _sessions.containsKey(id))
+            // A requested session ID can only be used if it is in the global map of
+            // ID but not in this contexts map.  Ie it is an ID in use by another context
+            // in this server and thus we are doing a cross context dispatch.
+            if (_crossContextSessionIDs)
+            {
+                String requested_id=(String)request.getAttribute(__NEW_SESSION_ID);
+                if (requested_id==null)
+                    requested_id=request.getRequestedSessionId();
+                if (requested_id !=null && 
+                    requested_id!=null && __allSessions.containsKey(requested_id) && !_sessions.containsKey(requested_id))
+                return requested_id;
+            }
+            
+            // pick a new unique ID!
+            String id=null;
+            while (id==null || id.length()==0 || __allSessions.containsKey(id))
             {
                 long r = _random.nextLong();
                 if (r<0)r=-r;
@@ -189,33 +237,70 @@ public abstract class AbstractSessionManager implements SessionManager
     /* ------------------------------------------------------------ */
     public HttpSession getHttpSession(String id)
     {
-        synchronized(_sessions)
+        synchronized(this)
         {
             return (HttpSession)_sessions.get(id);
         }
     }
-
+    
     /* ------------------------------------------------------------ */
     public HttpSession newHttpSession(HttpServletRequest request)
     {
         Session session = newSession(request);
         session.setMaxInactiveInterval(_dftMaxIdleSecs);
-        synchronized(_sessions)
+        synchronized(__allSessions)
         {
-            _sessions.put(session.getId(),session);
-            if (_sessions.size () > this._maxSessions)
-               this._maxSessions = _sessions.size ();
+            synchronized(this)
+            {
+              _sessions.put(session.getId(),session);
+              __allSessions.add(session.getId(), session);
+              if (_sessions.size() > this._maxSessions)
+                  this._maxSessions = _sessions.size ();
+            }
         }
         
         HttpSessionEvent event=new HttpSessionEvent(session);
         
         for(int i=0;i<_sessionListeners.size();i++)
             ((HttpSessionListener)_sessionListeners.get(i))
-                .sessionCreated(event);
+            .sessionCreated(event);
+        
+        if (getCrossContextSessionIDs())
+            request.setAttribute(__NEW_SESSION_ID, session.getId());
         return session;
     }
-    
 
+    /* ------------------------------------------------------------ */
+    public Cookie getSessionCookie(HttpSession session,boolean requestIsSecure)
+    {
+        if (_handler.isUsingCookies())
+        {
+            Cookie cookie = _handler.getSessionManager().getHttpOnly()
+                ?new HttpOnlyCookie(SessionManager.__SessionCookie,session.getId())
+                :new Cookie(SessionManager.__SessionCookie,session.getId());    
+            String domain=_handler.getServletContext().getInitParameter(SessionManager.__SessionDomain);
+            String maxAge=_handler.getServletContext().getInitParameter(SessionManager.__MaxAge);
+            String path=_handler.getServletContext().getInitParameter(SessionManager.__SessionPath);
+            if (path==null)
+                path=getCrossContextSessionIDs()?"/":_handler.getHttpContext().getContextPath();
+            if (path==null || path.length()==0)
+                path="/";
+            
+            if (domain!=null)
+                cookie.setDomain(domain);       
+            if (maxAge!=null)
+                cookie.setMaxAge(Integer.parseInt(maxAge));
+            else
+                cookie.setMaxAge(-1);
+            
+            cookie.setSecure(requestIsSecure && getSecureCookies());
+            cookie.setPath(path);
+            
+            return cookie;    
+        }
+        return null;
+    }
+    
     /* ------------------------------------------------------------ */
     protected abstract Session newSession(HttpServletRequest request);
     
@@ -229,7 +314,7 @@ public abstract class AbstractSessionManager implements SessionManager
     {
         return _workerName;
     }
-
+    
     /* ------------------------------------------------------------ */
     /** Set the workname.
      * If set, the workername is dot appended to the session ID
@@ -298,28 +383,68 @@ public abstract class AbstractSessionManager implements SessionManager
         }
     }
     
+    /* ------------------------------------------------------------ */
+    /**
+     * @return Returns the httpOnly.
+     */
+    public boolean getHttpOnly()
+    {
+        return _httpOnly;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @param httpOnly The httpOnly to set.
+     */
+    public void setHttpOnly(boolean httpOnly)
+    {
+        _httpOnly = httpOnly;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @return Returns the secureCookies.
+     */
+    public boolean getSecureCookies()
+    {
+        return _secureCookies;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @param secureCookies The secureCookies to set.
+     */
+    public void setSecureCookies(boolean secureCookies)
+    {
+        _secureCookies = secureCookies;
+    }
+
+    /* ------------------------------------------------------------ */
+    public boolean isInvalidateGlobal()
+    {
+        return _invalidateGlobal;
+    }
+    
+    /* ------------------------------------------------------------ */
+    /**
+     * @param global True if session invalidation should be global.
+     * ie Sessions in other contexts with the same ID (linked by cross context dispatch
+     * or shared session cookie) are invalidated as a group.
+     */
+    public void setInvalidateGlobal(boolean global)
+    {
+        _invalidateGlobal=global;
+    }
     
     /* ------------------------------------------------------------ */
     public void addEventListener(EventListener listener)
-        throws IllegalArgumentException
+    throws IllegalArgumentException
     {
         
-        // TODO - this needs to be checked.
-         
-        boolean known =false;
         if (listener instanceof HttpSessionAttributeListener)
-        {
             _sessionAttributeListeners.add(listener);
-            known=true;
-        }
         if (listener instanceof HttpSessionListener)
-        {
             _sessionListeners.add(listener);
-            known=true;
-        }
-
-        if (!known)
-            throw new IllegalArgumentException("Unknown listener "+listener);
     }
     
     /* ------------------------------------------------------------ */
@@ -339,7 +464,7 @@ public abstract class AbstractSessionManager implements SessionManager
     
     /* ------------------------------------------------------------ */
     public void start()
-        throws Exception
+    throws Exception
     {
         if (_random==null)
         {
@@ -391,9 +516,9 @@ public abstract class AbstractSessionManager implements SessionManager
         ClassLoader old_loader = thread.getContextClassLoader();
         try
         {
-	    if (_handler==null)
-		return;
-
+            if (_handler==null)
+                return;
+            
             ClassLoader loader = _handler.getClassLoader();
             if (loader!=null)
                 thread.setContextClassLoader(loader);
@@ -404,9 +529,10 @@ public abstract class AbstractSessionManager implements SessionManager
             // we build a list of stale sessions, then go back and invalidate them
             Object stale=null;
             
-            // For each session
-            try
+
+            synchronized(AbstractSessionManager.this)
             {
+                // For each session
                 for (Iterator i = _sessions.values().iterator(); i.hasNext(); )
                 {
                     Session session = (Session)i.next();
@@ -417,25 +543,7 @@ public abstract class AbstractSessionManager implements SessionManager
                     }
                 }
             }
-            catch(ConcurrentModificationException e)
-            {
-                LogSupport.ignore(log,e);
-                // Oops something changed while we were looking.
-                // Lock the context and try again.
-                // Set our priority high while we have the sessions locked
-                int oldPriority = Thread.currentThread().getPriority();
-                Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
-                try
-                {
-                    synchronized(this)
-                    {
-                        stale=null;
-                        scavenge();
-                    }
-                }
-                finally {Thread.currentThread().setPriority(oldPriority);}
-            }
-
+            
             // Remove the stale sessions
             for (int i = LazyList.size(stale); i-->0;)
             {
@@ -457,7 +565,7 @@ public abstract class AbstractSessionManager implements SessionManager
         }
     }
     
-
+    
     /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
     /* -------------------------------------------------------------- */
@@ -490,16 +598,16 @@ public abstract class AbstractSessionManager implements SessionManager
                 log.debug("Session scavenger exited");
             }
         }
-
+        
         SessionScavenger()
         {
             super("SessionScavenger");
             setDaemon(true);
         }
-
+        
     }   // SessionScavenger
-
-
+    
+    
     
     /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
@@ -513,7 +621,7 @@ public abstract class AbstractSessionManager implements SessionManager
         long _accessed=_created;
         long _maxIdleMs = _dftMaxIdleSecs*1000;
         String _id;
-
+        
         /* ------------------------------------------------------------- */
         protected Session(HttpServletRequest request)
         {
@@ -521,17 +629,17 @@ public abstract class AbstractSessionManager implements SessionManager
             if (_dftMaxIdleSecs>=0)
                 _maxIdleMs=_dftMaxIdleSecs*1000;
         }
-
+        
         /* ------------------------------------------------------------ */
         protected abstract Map newAttributeMap();
-
+        
         /* ------------------------------------------------------------ */
         public void access()
         {
             _newSession=false;
             _accessed=System.currentTimeMillis();
         }
-
+        
         /* ------------------------------------------------------------ */
         public boolean isValid()
         {
@@ -546,45 +654,45 @@ public abstract class AbstractSessionManager implements SessionManager
         
         /* ------------------------------------------------------------- */
         public String getId()
-            throws IllegalStateException
+        throws IllegalStateException
         {
             return _id;
         }
-
+        
         /* ------------------------------------------------------------- */
         public long getCreationTime()
-            throws IllegalStateException
+        throws IllegalStateException
         {
             if (_invalid) throw new IllegalStateException();
             return _created;
         }
-
+        
         /* ------------------------------------------------------------- */
         public long getLastAccessedTime()
-            throws IllegalStateException
+        throws IllegalStateException
         {
             if (_invalid) throw new IllegalStateException();
             return _accessed;
         }
-
+        
         /* ------------------------------------------------------------- */
         public int getMaxInactiveInterval()
         {
             if (_invalid) throw new IllegalStateException();
             return (int)(_maxIdleMs / 1000);
         }
-
+        
         /* ------------------------------------------------------------- */
         /**
          * @deprecated
          */
         public HttpSessionContext getSessionContext()
-            throws IllegalStateException
+        throws IllegalStateException
         {
             if (_invalid) throw new IllegalStateException();
             return SessionContext.NULL_IMPL;
         }
-
+        
         /* ------------------------------------------------------------- */
         public void setMaxInactiveInterval(int secs)
         {
@@ -592,71 +700,83 @@ public abstract class AbstractSessionManager implements SessionManager
             if (_maxIdleMs>0 && (_maxIdleMs/10)<_scavengePeriodMs)
                 AbstractSessionManager.this.setScavengePeriod((secs+9)/10);
         }
-
+        
         /* ------------------------------------------------------------- */
-        public synchronized void invalidate()
-            throws IllegalStateException
+        public void invalidate() throws IllegalStateException
         {
-            if (_invalid) throw new IllegalStateException();
-
-            
-            if(__24SessionDestroyed && _sessionListeners != null)
-            {        
-                HttpSessionEvent event = new HttpSessionEvent(this);
-                for(int i=0;i<_sessionListeners.size();i++)
-                    ((HttpSessionListener)_sessionListeners.get(i)).
-                        sessionDestroyed(event);
-            }
-                    
-            if (_values!=null)
+            if (log.isDebugEnabled()) log.debug("Invalidate session "+getId()+" in "+_handler.getHttpContext());
+            try
             {
-                Iterator iter = _values.keySet().iterator();
-                while (iter.hasNext())
+                // Notify listeners and unbind values
+                synchronized (this)
                 {
-                    String key = (String)iter.next();
-                    Object value = _values.get(key);
-                    iter.remove();
-                    unbindValue(key, value);
-                    
-                    if (_sessionAttributeListeners.size()>0)
+                    if (_invalid)
+                        throw new IllegalStateException();
+
+                    if (_sessionListeners!=null)
                     {
-                        HttpSessionBindingEvent event =
-                            new HttpSessionBindingEvent(this,key,value);
-                        
-                        for(int i=0;i<_sessionAttributeListeners.size();i++)
+                        HttpSessionEvent event=new HttpSessionEvent(this);
+                        for (int i=0; i<_sessionListeners.size(); i++)
+                            ((HttpSessionListener)_sessionListeners.get(i)).sessionDestroyed(event);
+                    }
+
+                    if (_values!=null)
+                    {
+                        Iterator iter=_values.keySet().iterator();
+                        while (iter.hasNext())
                         {
-                            ((HttpSessionAttributeListener)
-                             _sessionAttributeListeners.get(i))
-                                .attributeRemoved(event);
+                            String key=(String)iter.next();
+                            Object value=_values.get(key);
+                            iter.remove();
+                            unbindValue(key,value);
+
+                            if (_sessionAttributeListeners.size()>0)
+                            {
+                                HttpSessionBindingEvent event=new HttpSessionBindingEvent(this,key,value);
+
+                                for (int i=0; i<_sessionAttributeListeners.size(); i++)
+                                {
+                                    ((HttpSessionAttributeListener)_sessionAttributeListeners.get(i)).attributeRemoved(event);
+                                }
+                            }
                         }
                     }
                 }
             }
-            
-            synchronized (AbstractSessionManager.this)
+            finally
             {
-                _invalid=true;
-                _sessions.remove(_id);
-                
-                if(!__24SessionDestroyed && _sessionListeners != null)
+                // Remove session from context and global maps
+                synchronized (__allSessions)
                 {
-                    HttpSessionEvent event = new HttpSessionEvent(this);
-                    for(int i=0;i<_sessionListeners.size();i++)
-                        ((HttpSessionListener)_sessionListeners.get(i)).
-                            sessionDestroyed(event);       
+                    synchronized (_sessions)
+                    {
+                        _invalid=true;
+                        _sessions.remove(getId());
+                        __allSessions.removeValue(getId(), this);
+                        
+                        if (isInvalidateGlobal())
+                        {
+                            // Don't iterate as other sessions may also be globally invalidating
+                            while(__allSessions.containsKey(getId()))
+                            {
+                                Session session=(Session)__allSessions.getValue(getId(),0);
+                                session.invalidate();
+                            }
+                        }
+                    }
                 }
-            } 
+            }
         }
-
+        
         /* ------------------------------------------------------------- */
         public boolean isNew()
-            throws IllegalStateException
+        throws IllegalStateException
         {
             if (_invalid) throw new IllegalStateException();
             return _newSession;
         }
-
-
+        
+        
         /* ------------------------------------------------------------ */
         public synchronized Object getAttribute(String name)
         {
@@ -665,15 +785,15 @@ public abstract class AbstractSessionManager implements SessionManager
                 return null;
             return _values.get(name);
         }
-
+        
         /* ------------------------------------------------------------ */
         public synchronized Enumeration getAttributeNames()
         {
             if (_invalid) throw new IllegalStateException();
             List names = _values==null?Collections.EMPTY_LIST:new ArrayList(_values.keySet());
-	    return Collections.enumeration(names);
+            return Collections.enumeration(names);
         }
-
+        
         /* ------------------------------------------------------------ */
         public synchronized void setAttribute(String name, Object value)
         {
@@ -681,7 +801,7 @@ public abstract class AbstractSessionManager implements SessionManager
             if (_values==null)
                 _values=newAttributeMap();
             Object oldValue = _values.put(name,value);
-
+            
             if (value==null || !value.equals(oldValue))
             {
                 unbindValue(name, oldValue);
@@ -691,7 +811,7 @@ public abstract class AbstractSessionManager implements SessionManager
                 {
                     HttpSessionBindingEvent event =
                         new HttpSessionBindingEvent(this,name,
-                                                    oldValue==null?value:oldValue);
+                                oldValue==null?value:oldValue);
                     
                     for(int i=0;i<_sessionAttributeListeners.size();i++)
                     {
@@ -709,7 +829,7 @@ public abstract class AbstractSessionManager implements SessionManager
                 }
             }
         }
-
+        
         /* ------------------------------------------------------------ */
         public synchronized void removeAttribute(String name)
         {
@@ -736,25 +856,25 @@ public abstract class AbstractSessionManager implements SessionManager
                 }
             }
         }
-
+        
         /* ------------------------------------------------------------- */
         /**
          * @deprecated 	As of Version 2.2, this method is
          * 		replaced by {@link #getAttribute}
          */
         public Object getValue(String name)
-            throws IllegalStateException
+        throws IllegalStateException
         {
             return getAttribute(name);
         }
-
+        
         /* ------------------------------------------------------------- */
         /**
          * @deprecated 	As of Version 2.2, this method is
          * 		replaced by {@link #getAttributeNames}
          */
         public synchronized String[] getValueNames()
-            throws IllegalStateException
+        throws IllegalStateException
         {
             if (_invalid) throw new IllegalStateException();
             if (_values==null)
@@ -762,46 +882,48 @@ public abstract class AbstractSessionManager implements SessionManager
             String[] a = new String[_values.size()];
             return (String[])_values.keySet().toArray(a);
         }
-
+        
         /* ------------------------------------------------------------- */
         /**
          * @deprecated 	As of Version 2.2, this method is
          * 		replaced by {@link #setAttribute}
          */
         public void putValue(java.lang.String name,
-                             java.lang.Object value)
-            throws IllegalStateException
+                java.lang.Object value)
+        throws IllegalStateException
         {
             setAttribute(name,value);
         }
-
+        
         /* ------------------------------------------------------------- */
         /**
          * @deprecated 	As of Version 2.2, this method is
          * 		replaced by {@link #removeAttribute}
          */
         public void removeValue(java.lang.String name)
-            throws IllegalStateException
+        throws IllegalStateException
         {
             removeAttribute(name);
         }
-
+        
         /* ------------------------------------------------------------- */
         /** If value implements HttpSessionBindingListener, call valueBound() */
         private void bindValue(java.lang.String name, Object value)
         {
             if (value!=null && value instanceof HttpSessionBindingListener)
                 ((HttpSessionBindingListener)value)
-                    .valueBound(new HttpSessionBindingEvent(this,name));            
+                .valueBound(new HttpSessionBindingEvent(this,name));            
         }
-
+        
         /* ------------------------------------------------------------- */
         /** If value implements HttpSessionBindingListener, call valueUnbound() */
         private void unbindValue(java.lang.String name, Object value)
         {
             if (value!=null && value instanceof HttpSessionBindingListener)
                 ((HttpSessionBindingListener)value)
-                    .valueUnbound(new HttpSessionBindingEvent(this,name));
+                .valueUnbound(new HttpSessionBindingEvent(this,name));
         }
     }
+
+
 }
