@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: ServletHandler.java,v 1.110 2004/08/27 05:42:01 gregwilkins Exp $
+// $Id: ServletHandler.java,v 1.132 2005/12/04 06:26:04 janb Exp $
 // Copyright 199-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,6 +26,7 @@ import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 
@@ -39,17 +40,18 @@ import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 import org.mortbay.http.EOFException;
 import org.mortbay.http.HttpContext;
 import org.mortbay.http.HttpException;
 import org.mortbay.http.HttpFields;
+import org.mortbay.http.HttpHandler;
 import org.mortbay.http.HttpRequest;
 import org.mortbay.http.HttpResponse;
 import org.mortbay.http.PathMap;
 import org.mortbay.http.Version;
-import org.mortbay.http.handler.AbstractHttpHandler;
 import org.mortbay.util.ByteArrayISO8859Writer;
+import org.mortbay.util.Container;
 import org.mortbay.util.LogSupport;
 import org.mortbay.util.MultiException;
 import org.mortbay.util.Resource;
@@ -70,10 +72,10 @@ import org.mortbay.util.URI;
  * java.util.Random generator is created.
  * <P>
  * @see org.mortbay.jetty.servlet.WebApplicationHandler
- * @version $Id: ServletHandler.java,v 1.110 2004/08/27 05:42:01 gregwilkins Exp $
+ * @version $Id: ServletHandler.java,v 1.132 2005/12/04 06:26:04 janb Exp $
  * @author Greg Wilkins
  */
-public class ServletHandler extends AbstractHttpHandler
+public class ServletHandler extends Container implements HttpHandler
 {
     private static Log log = LogFactory.getLog(ServletHandler.class);
 
@@ -95,6 +97,7 @@ public class ServletHandler extends AbstractHttpHandler
     /* ------------------------------------------------------------ */
     private boolean _usingCookies=true;
     private boolean _autoInitializeServlets=true;
+    private String _name;
     
     /* ------------------------------------------------------------ */
     protected PathMap _servletMap=new PathMap();
@@ -107,19 +110,50 @@ public class ServletHandler extends AbstractHttpHandler
     protected transient Context _context;
     protected transient ClassLoader _loader;
     protected transient Log _contextLog;
+    protected transient HttpContext _httpContext;
 
     /* ------------------------------------------------------------ */
     /** Constructor. 
      */
     public ServletHandler()
-    {}
+    {
+        _context=new Context();
+    }
+
+    /* ------------------------------------------------------------ */
+    public void setName(String name)
+    {
+        _name=name;
+    }
+    
+    /* ------------------------------------------------------------ */
+    public String getName()
+    {
+        if (_name==null)
+        {
+            _name=this.getClass().getName();
+            if (!log.isDebugEnabled())
+                _name=_name.substring(_name.lastIndexOf('.')+1);
+        }
+        return _name;
+    }
+    
+    /* ------------------------------------------------------------ */
+    public HttpContext getHttpContext()
+    {
+        return _httpContext;
+    }
     
     /* ------------------------------------------------------------ */
     public void initialize(HttpContext context)
     {
         SessionManager sessionManager=getSessionManager();
-        super.initialize(context);
-        _context=new Context();
+        
+
+        if (_httpContext!=null&& _httpContext!=context)
+            throw new IllegalStateException("Can't initialize handler for different context");
+        _httpContext=context;
+        
         sessionManager.initialize(this);
     }
 
@@ -146,6 +180,7 @@ public class ServletHandler extends AbstractHttpHandler
             setMii=true;
             if (getHttpContext()!=null)
                 _sessionManager.initialize(null);
+            removeComponent(_sessionManager);
         }
 
         _sessionManager=sm;
@@ -156,6 +191,7 @@ public class ServletHandler extends AbstractHttpHandler
                 _sessionManager.initialize(this);
             if (setMii)
                 _sessionManager.setMaxInactiveInterval(mii);
+            addComponent(_sessionManager);
         } 
         
         _sessionManager=sm;
@@ -165,7 +201,10 @@ public class ServletHandler extends AbstractHttpHandler
     public SessionManager getSessionManager()
     {
         if (_sessionManager==null)
+        {
             _sessionManager = new HashSessionManager();
+            addComponent(_sessionManager);
+        }
         return _sessionManager;
     }
     
@@ -229,7 +268,8 @@ public class ServletHandler extends AbstractHttpHandler
             throw new IllegalArgumentException("Named servlet already exists: "+name);
         
         ServletHolder holder = new ServletHolder(this,name,servletClass,forcedPath);
-        _nameMap.put(holder.getName(),holder);
+        addServletHolder(holder);
+        
         return holder;
     }
 
@@ -247,6 +287,12 @@ public class ServletHandler extends AbstractHttpHandler
     }    
 
     /* ------------------------------------------------------------ */
+    /** 
+     * Map a servlet to a pathSpec
+     * @param pathSpec The pathspec to map
+     * @param servletName The name of the servlet, which must have already been added.
+     * @return The servlet holder of the mapped servlet.
+     */
     public ServletHolder mapPathToServlet(String pathSpec,
                                           String servletName)
     {
@@ -307,32 +353,32 @@ public class ServletHandler extends AbstractHttpHandler
 
     
     /* ------------------------------------------------------------ */
+    /**
+     * Add a servlet instance to this handler and map it to a pathspec.
+     * @param pathSpec The pathmapping
+     * @param servletClass The class of the servlet
+     * @return The created ServletHolder
+     */
     public ServletHolder addServlet(String pathSpec,
                                     String servletClass)
     {
         return addServlet(servletClass,pathSpec,servletClass,null);
     }
-
+    
     /* ------------------------------------------------------------ */
-    void addServletHolder(String pathSpec, ServletHolder holder)
+    /**
+     * Register an existing ServletHolder with this handler.
+     * @param holder the ServletHolder to register.
+     */
+    public void addServletHolder(ServletHolder holder)
     {
-        try
-        {
-            ServletHolder existing = (ServletHolder)
-                _nameMap.get(holder.getName());
-            if (existing==null)
-                _nameMap.put(holder.getName(),holder);
-            else if (existing!=holder)
-                throw new IllegalArgumentException("Holder already exists for name: "+holder.getName());
-            
-            if (isStarted() && !holder.isStarted())
-                holder.start();
-            _servletMap.put(pathSpec,holder);
-        }
-        catch(Exception e)
-        {
-            log.warn(LogSupport.EXCEPTION,e);
-        }
+        ServletHolder existing = (ServletHolder)
+        _nameMap.get(holder.getName());
+        if (existing==null)
+            _nameMap.put(holder.getName(),holder);
+        else if (existing!=holder)
+            throw new IllegalArgumentException("Holder already exists for name: "+holder.getName());
+        addComponent(holder);
     }
     
     /* ------------------------------------------------------------ */
@@ -348,13 +394,14 @@ public class ServletHandler extends AbstractHttpHandler
     }
     
     /* ----------------------------------------------------------------- */
-    public synchronized void start()
+    protected synchronized void doStart()
         throws Exception
     {
         if (isStarted())
             return;
         
-        _contextLog = LogFactory.getLog(getHttpContext().getHttpContextName());
+        _contextLog = LogFactory.getLog("org.mortbay.jetty.context."+getHttpContext().getHttpContextName());
+        
         if (_contextLog==null)
             _contextLog=log;
         
@@ -364,10 +411,6 @@ public class ServletHandler extends AbstractHttpHandler
         // Initialize classloader
         _loader=getHttpContext().getClassLoader();
 
-        // start the handler - protected by synchronization until
-        // end of the call.
-        super.start();
-
         if (_autoInitializeServlets)
             initializeServlets();
     }   
@@ -411,14 +454,12 @@ public class ServletHandler extends AbstractHttpHandler
     }
     
     /* ----------------------------------------------------------------- */
-    public synchronized void stop()
-        throws InterruptedException
+    protected synchronized void doStop()
+        throws Exception
     {
         // Sort and Initialize servlets
         ServletHolder[] holders = getServlets();
         
-        super.stop();
-        
         // Stop servlets
         for (int i=holders.length; i-->0;)
         {
@@ -437,19 +478,23 @@ public class ServletHandler extends AbstractHttpHandler
     }
 
     /* ------------------------------------------------------------ */
-    HttpSession getHttpSession(String id)
+    public HttpSession getHttpSession(String id)
     {
         return _sessionManager.getHttpSession(id);
     }
     
     /* ------------------------------------------------------------ */
-    HttpSession newHttpSession(HttpServletRequest request)
+    public HttpSession newHttpSession(HttpServletRequest request)
     {
         return _sessionManager.newHttpSession(request);
     }
 
     /* ------------------------------------------------------------ */
-    void setSessionInactiveInterval(int seconds)
+    /**
+     * Set the session timeout interval in seconds.
+     * @param seconds the length of the session timeout interval in seconds.
+     */
+    public void setSessionInactiveInterval(int seconds)
     {
         _sessionManager.setMaxInactiveInterval(seconds);
     }
@@ -468,7 +513,7 @@ public class ServletHandler extends AbstractHttpHandler
                        HttpResponse httpResponse)
          throws IOException
     {
-        if (!isStarted() && _context==null)
+        if (!isStarted())
             return;
         
         // Handle TRACE
@@ -478,12 +523,12 @@ public class ServletHandler extends AbstractHttpHandler
             return;
         }
 
-        // Look for existing request/response objects
+        // Look for existing request/response objects (from enterScope call)
         ServletHttpRequest request = (ServletHttpRequest) httpRequest.getWrapper();
         ServletHttpResponse response = (ServletHttpResponse) httpResponse.getWrapper();
         if (request==null)
         {
-            // Build the request and response.
+            // Not in ServletHttpContext, but bumble on anyway
             request = new ServletHttpRequest(this,pathInContext,httpRequest);
             response = new ServletHttpResponse(request,httpResponse);
             httpRequest.setWrapper(request);
@@ -491,12 +536,10 @@ public class ServletHandler extends AbstractHttpHandler
         }
         else
         {
-            // Recycled request
             request.recycle(this,pathInContext);
             response.recycle();
         }
         
-        
         // Look for the servlet
         Map.Entry servlet=getHolderEntry(pathInContext);
         ServletHolder servletHolder=servlet==null?null:(ServletHolder)servlet.getValue();
@@ -514,7 +557,7 @@ public class ServletHandler extends AbstractHttpHandler
             }
             
             // Handle the session ID
-            request.setSessionId(pathParams);
+            request.setRequestedSessionId(pathParams);
             HttpSession session=request.getSession(false);
             if (session!=null)
                 ((SessionManager.Session)session).access();
@@ -522,24 +565,19 @@ public class ServletHandler extends AbstractHttpHandler
             
             // Do that funky filter and servlet thang!
             if (servletHolder!=null)
-                dispatch(pathInContext,request,response,servletHolder);
+                dispatch(pathInContext,request,response,servletHolder, Dispatcher.__REQUEST);
         }
         catch(Exception e)
         {
             log.debug(LogSupport.EXCEPTION,e);
             
             Throwable th=e;
-            if (e instanceof ServletException)
+            while (th instanceof ServletException)
             {
-                Throwable root=((ServletException)e).getRootCause();
-                while (root instanceof ServletException)
-                    root=((ServletException)e).getRootCause();
-                if (root instanceof HttpException ||
-                    root instanceof EOFException)
-                {
-                    if(log.isDebugEnabled())log.debug("Extracting root cause from ",e);
-                    th=root;
-                }
+                Throwable cause=((ServletException)th).getRootCause();
+                if (cause==th || cause==null)
+                    break;
+                th=cause;
             }
             
             if (th instanceof HttpException)
@@ -561,10 +599,16 @@ public class ServletHandler extends AbstractHttpHandler
             {
                 request.setAttribute(ServletHandler.__J_S_ERROR_EXCEPTION_TYPE,th.getClass());
                 request.setAttribute(ServletHandler.__J_S_ERROR_EXCEPTION,th);
-                response.sendError(th instanceof UnavailableException
-                                   ?HttpResponse.__503_Service_Unavailable
-                                   :HttpResponse.__500_Internal_Server_Error,
-                                   e.getMessage());
+                if (th instanceof UnavailableException)
+                {
+                    UnavailableException ue = (UnavailableException)th;
+                    if (ue.isPermanent())
+                        response.sendError(HttpResponse.__404_Not_Found,e.getMessage());
+                    else
+                        response.sendError(HttpResponse.__503_Service_Unavailable,e.getMessage());
+                }
+                else
+                    response.sendError(HttpResponse.__500_Internal_Server_Error,e.getMessage());
             }
             else
                 if(log.isDebugEnabled())log.debug("Response already committed for handling "+th);
@@ -587,11 +631,9 @@ public class ServletHandler extends AbstractHttpHandler
         }
         finally
         {
-            if (servletHolder!=null)
+            if (servletHolder!=null && response!=null)
             {
-                response.flushBuffer();
-                if (!httpRequest.isHandled())
-                    new Throwable().printStackTrace();
+                response.complete();
             }
         }
     }
@@ -607,6 +649,7 @@ public class ServletHandler extends AbstractHttpHandler
      * @param request 
      * @param response 
      * @param servletHolder 
+     * @param type the type of dispatch as defined in the Dispatcher class.
      * @exception ServletException 
      * @exception UnavailableException 
      * @exception IOException 
@@ -614,7 +657,8 @@ public class ServletHandler extends AbstractHttpHandler
     protected void dispatch(String pathInContext,
                             HttpServletRequest request,
                             HttpServletResponse response,
-                            ServletHolder servletHolder)
+                            ServletHolder servletHolder, 
+                            int type)
         throws ServletException,
                UnavailableException,
                IOException
@@ -673,6 +717,9 @@ public class ServletHandler extends AbstractHttpHandler
     public URL getResource(String uriInContext)
         throws MalformedURLException
     {        
+        if (uriInContext==null || !uriInContext.startsWith("/"))
+            throw new MalformedURLException(uriInContext);
+        
         try{
             Resource resource = getHttpContext().getResource(uriInContext);
             if (resource!=null && resource.exists())
@@ -696,20 +743,19 @@ public class ServletHandler extends AbstractHttpHandler
     /* ------------------------------------------------------------ */
     public InputStream getResourceAsStream(String uriInContext)
     {
+        if (uriInContext==null || !uriInContext.startsWith("/"))
+            return null;
         try
         {
             Resource resource = getHttpContext().getResource(uriInContext);
             if (resource!=null)
-            {
                 return resource.getInputStream();
-            }
             
             uriInContext=URI.canonicalPath(uriInContext);
             URL url = getResource(uriInContext);
             if (url!=null)
                 return url.openStream();
         }
-        catch(MalformedURLException e) {LogSupport.ignore(log,e);}
         catch(IOException e) {LogSupport.ignore(log,e);}
         return null;
     }
@@ -749,7 +795,7 @@ public class ServletHandler extends AbstractHttpHandler
             return null;
 
         if (!uriInContext.startsWith("/"))
-            uriInContext="/"+uriInContext;
+            return null;
         
         try
         {
@@ -799,7 +845,7 @@ public class ServletHandler extends AbstractHttpHandler
 
     
     /* ------------------------------------------------------------ */
-    void notFound(HttpServletRequest request,
+    protected void notFound(HttpServletRequest request,
                   HttpServletResponse response)
         throws IOException
     {
@@ -826,7 +872,7 @@ public class ServletHandler extends AbstractHttpHandler
     }
     
     /* ------------------------------------------------------------ */
-    void handleTrace(HttpServletRequest request,
+    protected void handleTrace(HttpServletRequest request,
                             HttpServletResponse response)
         throws IOException
     {
@@ -842,7 +888,7 @@ public class ServletHandler extends AbstractHttpHandler
     }
     
     /* ------------------------------------------------------------ */
-    void handleOptions(HttpServletRequest request,
+    protected void handleOptions(HttpServletRequest request,
                               HttpServletResponse response)
         throws IOException
     {
@@ -859,12 +905,115 @@ public class ServletHandler extends AbstractHttpHandler
     }
 
     /* ------------------------------------------------------------ */
-    String getErrorPage(int status,ServletHttpRequest request)
+    public String getErrorPage(int status,ServletHttpRequest request)
     {
         return null;
     }
     
+    /* ------------------------------------------------------------ */
+    /** Get context attribute.
+     * Tries ServletHandler attributes and then delegated to HttpContext.
+     * @param name attribute name.
+     * @return attribute
+     */
+    protected Object getContextAttribute(String name)
+    {
+        if (ServletHandler.__J_S_CONTEXT_TEMPDIR.equals(name))
+        {
+            // Initialize temporary directory
+            Object t = getHttpContext().getAttribute(ServletHandler.__J_S_CONTEXT_TEMPDIR);
+
+            if (t instanceof File)
+                return (File)t;
+            
+            return getHttpContext().getTempDirectory();
+        }
+
+        if (_attributes.containsKey(name))
+            return _attributes.get(name);
+        return getHttpContext().getAttribute(name);
+    }
+
+    /* ------------------------------------------------------------ */
+    /** Get context attribute names.
+     * Combines ServletHandler and HttpContext attributes.
+     */
+    protected Enumeration getContextAttributeNames()
+    {
+        if (_attributes.size()==0)
+            return getHttpContext().getAttributeNames();
+        HashSet set=new HashSet(_attributes.keySet());
+        Enumeration e=getHttpContext().getAttributeNames();
+        while(e.hasMoreElements())
+            set.add(e.nextElement());
+        return Collections.enumeration(set);
+    }
+
+    /* ------------------------------------------------------------ */
+    /* Set a Servlet context attribute.
+     * Servlet Context attributes may hide HttpContext attributes.
+     */
+    protected void setContextAttribute(String name, Object value)
+    {
+        _attributes.put(name,value);
+    }
     
+    /* ------------------------------------------------------------ */
+    /* Remove a Servlet context attribute.
+     * Servlet Context attributes may hide HttpContext attributes.
+     */
+    protected void removeContextAttribute(String name)
+    {
+        _attributes.remove(name);
+    }
+    
+
+    /* ----------------------------------------------------------------- */
+    public void handleTrace(HttpRequest request,
+                            HttpResponse response)
+        throws IOException
+    {
+        boolean trace=getHttpContext().getHttpServer().getTrace();
+        
+        // Handle TRACE by returning request header
+        response.setField(HttpFields.__ContentType,
+                          HttpFields.__MessageHttp);
+        if (trace)
+        {
+            OutputStream out = response.getOutputStream();
+            ByteArrayISO8859Writer writer = new ByteArrayISO8859Writer();
+            writer.write(request.toString());
+            writer.flush();
+            response.setIntField(HttpFields.__ContentLength,writer.size());
+            writer.writeTo(out);
+            out.flush();
+        }
+        request.setHandled(true);
+    }
+    
+    /* ----------------------------------------------------------------- */
+    public void destroy()
+    {
+        Iterator iter = _nameMap.values().iterator();
+        while (iter.hasNext())
+        {
+            Object sh=iter.next();
+            iter.remove();
+            removeComponent(sh);
+        }
+        
+        if (_sessionManager!=null)
+            removeComponent(_sessionManager);
+        _sessionManager=null;
+        _context=null;
+        super.destroy();
+    }
+    
+    /* ----------------------------------------------------------------- */
+    protected void finalize() throws Throwable
+    {
+        destroy();
+    }
     
     /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
@@ -1005,7 +1154,7 @@ public class ServletHandler extends AbstractHttpHandler
         /* ------------------------------------------------------------ */
         public String getServerInfo()
         {
-            return Version.__VersionImpl;
+            return Version.getImplVersion();
         }
 
 
@@ -1039,20 +1188,7 @@ public class ServletHandler extends AbstractHttpHandler
          */
         public Object getAttribute(String name)
         {
-            if (ServletHandler.__J_S_CONTEXT_TEMPDIR.equals(name))
-            {
-                // Initialize temporary directory
-                Object t = getHttpContext().getAttribute(ServletHandler.__J_S_CONTEXT_TEMPDIR);
-
-                if (t instanceof File)
-                    return (File)t;
-                
-                return getHttpContext().getTempDirectory();
-            }
-
-            if (_attributes.containsKey(name))
-                return _attributes.get(name);
-            return getHttpContext().getAttribute(name);
+            return getContextAttribute(name);
         }
 
         /* ------------------------------------------------------------ */
@@ -1061,13 +1197,7 @@ public class ServletHandler extends AbstractHttpHandler
          */
         public Enumeration getAttributeNames()
         {
-            if (_attributes.size()==0)
-                return getHttpContext().getAttributeNames();
-            HashSet set=new HashSet(_attributes.keySet());
-            Enumeration e=getHttpContext().getAttributeNames();
-            while(e.hasMoreElements())
-                set.add(e.nextElement());
-            return Collections.enumeration(set);
+            return getContextAttributeNames();
         }
 
         /* ------------------------------------------------------------ */
@@ -1078,7 +1208,7 @@ public class ServletHandler extends AbstractHttpHandler
          */
         public void setAttribute(String name, Object value)
         {
-            _attributes.put(name,value);
+            setContextAttribute(name, value);
         }
 
         /* ------------------------------------------------------------ */
@@ -1088,7 +1218,7 @@ public class ServletHandler extends AbstractHttpHandler
          */
         public void removeAttribute(String name)
         {
-            _attributes.put(name, null);
+            removeContextAttribute(name);
         }
     
         /* ------------------------------------------------------------ */
