@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: ServletHttpResponse.java,v 1.48 2004/05/09 20:32:27 gregwilkins Exp $
+// $Id: ServletHttpResponse.java,v 1.64 2005/10/07 11:52:59 gregwilkins Exp $
 // Copyright 2000-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -29,7 +29,7 @@ import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 import org.mortbay.http.HttpContext;
 import org.mortbay.http.HttpFields;
 import org.mortbay.http.HttpOutputStream;
@@ -49,7 +49,7 @@ import org.mortbay.util.URI;
  * be operated on by multiple threads, then higher level
  * synchronizations may be required.
  *
- * @version $Id: ServletHttpResponse.java,v 1.48 2004/05/09 20:32:27 gregwilkins Exp $
+ * @version $Id: ServletHttpResponse.java,v 1.64 2005/10/07 11:52:59 gregwilkins Exp $
  * @author Greg Wilkins (gregw)
  */
 public class ServletHttpResponse implements HttpServletResponse
@@ -88,10 +88,11 @@ public class ServletHttpResponse implements HttpServletResponse
     private HttpSession _session=null;
     private boolean _noSession=false;
     private Locale _locale=null;
+    private boolean _explicitEncoding=false;
 
     
     /* ------------------------------------------------------------ */
-    ServletHttpResponse(ServletHttpRequest request,HttpResponse response)
+    public ServletHttpResponse(ServletHttpRequest request,HttpResponse response)
     {
         _servletHttpRequest=request;
         _servletHttpRequest.setServletHttpResponse(this);
@@ -107,6 +108,7 @@ public class ServletHttpResponse implements HttpServletResponse
         _session=null;
         _noSession=false;
         _locale=null;
+        _explicitEncoding=false;
     }
     
     /* ------------------------------------------------------------ */
@@ -144,7 +146,19 @@ public class ServletHttpResponse implements HttpServletResponse
     void commit()
         throws IOException
     {
-        _httpResponse.commit();
+        if (_writer!=null && _writer.isWritten())
+            _writer.flush();
+        else
+            _httpResponse.commit();
+    }
+    
+    /* ------------------------------------------------------------ */
+    void complete()
+        throws IOException
+    {
+        _httpResponse.completing();
+        commit();
+        setOutputState(DISABLED);
     }
 
     /* ------------------------------------------------------------ */
@@ -195,10 +209,10 @@ public class ServletHttpResponse implements HttpServletResponse
     public void resetBuffer()
     {
         if (isCommitted())
-            throw new IllegalStateException("committed");
+            throw new IllegalStateException("Committed");
+        
         ((HttpOutputStream)_httpResponse.getOutputStream()).resetBuffer();
-        if (_writer!=null)
-            _writer.reset();
+        if (_writer!=null) _writer.reset();
     }
     
     /* ------------------------------------------------------------ */
@@ -220,32 +234,23 @@ public class ServletHttpResponse implements HttpServletResponse
      */
     public void setLocale(Locale locale)
     {
-        if (locale == null)
+        if (locale == null || isCommitted())
             return; 
 
         _locale = locale;
         setHeader(HttpFields.__ContentLanguage,locale.toString().replace('_','-'));
-                          
-        /* get current MIME type from Content-Type header */                  
-        String type=_httpResponse.getField(HttpFields.__ContentType);
-        if (type==null)
-        {
-            // servlet did not set Content-Type yet
-            // so lets assume default one
-            type="application/octet-stream";
-        }
-        else if (type.startsWith("text/") && _httpResponse.getCharacterEncoding()==null)
-            /* If there is already charset parameter in content-type,
-               we will leave it alone as it is already correct.
-               This allows for both setContentType() and setLocale() to be called.
-               It makes some sense for text/ MIME types to try to guess output encoding 
-               based on language code from locale when charset parameter is not present.
-               Guessing is not a problem because when encoding matters, setContentType()
-               should be called with charset parameter in MIME type.
-              
-               JH: I think guessing should be exterminated as it makes no sense.
-            */
+                         
+        if (this._outputState==0)
         {
+            /* get current MIME type from Content-Type header */                  
+            String type=_httpResponse.getField(HttpFields.__ContentType);
+            if (type==null)
+            {
+                // servlet did not set Content-Type yet
+                // so lets assume default one
+                type="application/octet-stream";
+            }
+            
             HttpContext httpContext=_servletHttpRequest.getServletHandler().getHttpContext();
             if (httpContext instanceof ServletHttpContext)
             {
@@ -255,19 +260,21 @@ public class ServletHttpResponse implements HttpServletResponse
                     int semi=type.indexOf(';');
                     if (semi<0)
                         type += "; charset="+charset;
-                    else
+                    else if (!_explicitEncoding)
                         type = type.substring(0,semi)+"; charset="+charset;
+                    
+                    setHeader(HttpFields.__ContentType,type);
                 }
+                
             }
         }
-        
-        /* lets put updated MIME type back */
-        setHeader(HttpFields.__ContentType,type);
     }
     
     /* ------------------------------------------------------------ */
     public Locale getLocale()
     {
+        if (_locale==null)
+            return Locale.getDefault();
         return _locale;
     }
     
@@ -368,6 +375,8 @@ public class ServletHttpResponse implements HttpServletResponse
         String error_page =
             _servletHttpRequest.getServletHandler().getErrorPage(status,_servletHttpRequest);
 
+        resetBuffer();
+        
         // Handle error page?
         if (error_page==null)
         {
@@ -375,7 +384,9 @@ public class ServletHttpResponse implements HttpServletResponse
             _httpResponse.sendError(status,message);
         }
         else
-        {
+        {   
+            _httpResponse.setStatus(status,message);
+            
             if (message == null)
             {
                 message= (String)HttpResponse.__statusMsg.get(TypeUtil.newInteger(status));
@@ -409,13 +420,14 @@ public class ServletHttpResponse implements HttpServletResponse
                 _httpResponse.sendError(status,message); 
             }
         }
+        complete();
     }
 
     /* ------------------------------------------------------------ */
     public void sendError(int status) 
         throws IOException
     {
-	sendError(status,null);
+        sendError(status,null);
     }
 
     /* ------------------------------------------------------------ */
@@ -442,7 +454,12 @@ public class ServletHttpResponse implements HttpServletResponse
             
             url=buf.toString();
         }
-        _httpResponse.sendRedirect(url);
+        
+        resetBuffer();
+        
+        _httpResponse.setField(HttpFields.__Location,url);
+        _httpResponse.setStatus(HttpResponse.__302_Moved_Temporarily);
+        complete();
     }
 
     /* ------------------------------------------------------------ */
@@ -554,17 +571,21 @@ public class ServletHttpResponse implements HttpServletResponse
         if (_writer==null)
         {
             /* get encoding from Content-Type header */
-            String encoding = getCharacterEncoding();
-            if (encoding==null && _servletHttpRequest!=null)
-            {
-                /* implementation of educated defaults */
-                String mimeType = _httpResponse.getMimeType();                
-                encoding = _servletHttpRequest.getServletHandler()
-                    .getHttpContext().getEncodingByMimeType(mimeType);
-            }
+            String encoding = _httpResponse.getCharacterEncoding();
+            
             if (encoding==null)
-                // get last resort hardcoded default
-                encoding = StringUtil.__ISO_8859_1; 
+            {
+                if (_servletHttpRequest!=null)
+                {
+                    /* implementation of educated defaults */
+                    String mimeType = _httpResponse.getMimeType();                
+                    encoding = _servletHttpRequest.getServletHandler()
+                    	.getHttpContext().getEncodingByMimeType(mimeType);
+                }
+                if (encoding==null)
+                    encoding = StringUtil.__ISO_8859_1; 
+                _httpResponse.setCharacterEncoding(encoding,true);
+            }
             
             /* construct Writer using correct encoding */
             _writer = new ServletWriter(_httpResponse.getOutputStream(), encoding);
@@ -592,7 +613,23 @@ public class ServletHttpResponse implements HttpServletResponse
     /* ------------------------------------------------------------ */
     public void setContentType(String contentType) 
     {
-        _httpResponse.setContentType(contentType);
+        if (isCommitted() || contentType==null)
+            return;
+        
+        int semi=contentType.indexOf(';');
+        if (semi>0)
+        {
+            if (_outputState==0)
+            {
+                _explicitEncoding=true;
+                _httpResponse.setContentType(contentType);
+            }
+            else
+                _httpResponse.setContentType(contentType.substring(0,semi));
+        }
+        else
+            _httpResponse.setContentType(contentType);
+        
         if (_locale!=null)
             setLocale(_locale);
     }
@@ -600,7 +637,11 @@ public class ServletHttpResponse implements HttpServletResponse
     /* ------------------------------------------------------------ */
     public void setCharacterEncoding(String encoding)
     {
-        _httpResponse.setCharacterEncoding(encoding,true);
+        if (this._outputState==0 && !isCommitted())
+        {
+            _explicitEncoding=true;
+            _httpResponse.setCharacterEncoding(encoding,true);
+        }
     }
     
     /* ------------------------------------------------------------ */
