@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: Dump.java,v 1.34 2004/05/09 20:32:41 gregwilkins Exp $
+// $Id: Dump.java,v 1.42 2005/12/21 23:14:38 gregwilkins Exp $
 // Copyright 1996-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -33,7 +33,7 @@ import javax.servlet.http.HttpServletRequestWrapper;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 import org.mortbay.html.Break;
 import org.mortbay.html.Font;
 import org.mortbay.html.Heading;
@@ -72,6 +72,9 @@ public class Dump extends HttpServlet
     public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
     {
         request.setAttribute("Dump", this);
+        request.setCharacterEncoding("ISO_8859_1");
+        getServletContext().setAttribute("Dump",this);
+        
         String info= request.getPathInfo();
         if (info != null && info.endsWith("Exception"))
         {
@@ -88,10 +91,27 @@ public class Dump extends HttpServlet
         String redirect= request.getParameter("redirect");
         if (redirect != null && redirect.length() > 0)
         {
+            response.getOutputStream().println("THIS SHOULD NOT BE SEEN!");
             response.sendRedirect(redirect);
+            response.getOutputStream().println("THIS SHOULD NOT BE SEEN!");
             return;
         }
 
+        String error= request.getParameter("error");
+        if (error != null && error.length() > 0)
+        {
+            response.getOutputStream().println("THIS SHOULD NOT BE SEEN!");
+            response.sendError(Integer.parseInt(error));
+            response.getOutputStream().println("THIS SHOULD NOT BE SEEN!");
+            return;
+        }
+
+        String length= request.getParameter("length");
+        if (length != null && length.length() > 0)
+        {
+            response.setContentLength(Integer.parseInt(length));
+        }
+
         String buffer= request.getParameter("buffer");
         if (buffer != null && buffer.length() > 0)
             response.setBufferSize(Integer.parseInt(buffer));
@@ -114,16 +134,19 @@ public class Dump extends HttpServlet
             }
         }
 
-        if (info != null && info.indexOf("Cookie") >= 0)
+        String cn= request.getParameter("cookie");
+        String cv=request.getParameter("value");
+        String v=request.getParameter("version");
+        if (cn!=null && cv!=null)
         {
-            Cookie cookie= new Cookie("Cookie", info);
-            cookie.setMaxAge(300);
-            cookie.setPath("/");
+            Cookie cookie= new Cookie(cn, cv);
             cookie.setComment("Cookie from dump servlet");
-            if (info.indexOf("Cookie1") >= 0)
-                cookie.setVersion(1);
-            if (info.indexOf("Cookie2") >= 0)
-                cookie.setVersion(2);
+            if (v!=null)
+            {
+                cookie.setMaxAge(300);
+                cookie.setPath("/");
+                cookie.setVersion(Integer.parseInt(v));
+            }
             response.addCookie(cookie);
         }
 
@@ -163,6 +186,9 @@ public class Dump extends HttpServlet
             table.addHeading("getContentType:&nbsp;").cell().right();
             table.addCell("" + request.getContentType());
             table.newRow();
+            table.addHeading("getCharacterEncoding:&nbsp;").cell().right();
+            table.addCell("" + request.getCharacterEncoding());
+            table.newRow();
             table.addHeading("getRequestURI:&nbsp;").cell().right();
             table.addCell("" + request.getRequestURI());
             table.newRow();
@@ -298,6 +324,24 @@ public class Dump extends HttpServlet
                     }
             }
 
+            table.newRow();
+            table
+                .newHeading()
+                .cell()
+                .nest(new Font(2, true))
+                .add("<BR>Cookies")
+                .attribute("COLSPAN", "2")
+                .left();
+            Cookie[] cookies = request.getCookies();
+            for (int i=0; cookies!=null && i<cookies.length;i++)
+            {
+                Cookie cookie = cookies[i];
+
+                table.newRow();
+                table.addHeading(cookie.getName() + ":&nbsp;").cell().attribute("VALIGN", "TOP").right();
+                table.addCell(cookie.getValue());
+            }
+            
             /* ------------------------------------------------------------ */
             table.newRow();
             table
@@ -418,7 +462,8 @@ public class Dump extends HttpServlet
 
                 table.newRow();
                 table.addHeading("getServletContext():&nbsp;").cell().right();
-                table.addCell("" + getServletContext().getResource(res));
+                try{table.addCell("" + getServletContext().getResource(res));}
+                catch(Exception e) {table.addCell("" +e);}
             }
             
 
