@@ -1,5 +1,5 @@
 // ========================================================================
-// $Id: UrlEncoded.java,v 1.20 2004/05/09 20:33:05 gregwilkins Exp $
+// $Id: UrlEncoded.java,v 1.24 2005/12/21 23:14:38 gregwilkins Exp $
 // Copyright 1999-2004 Mort Bay Consulting Pty. Ltd.
 // ------------------------------------------------------------------------
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,7 +20,7 @@ import java.util.Iterator;
 import java.util.Map;
 
 import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.mortbay.log.LogFactory;
 
 /* ------------------------------------------------------------ */
 /** Handles coding of MIME  "x-www-form-urlencoded".
@@ -35,7 +35,7 @@ import org.apache.commons.logging.LogFactory;
  * get operations are not protected from concurrent updates.
  *
  * @see java.net.URLEncoder
- * @version $Id: UrlEncoded.java,v 1.20 2004/05/09 20:33:05 gregwilkins Exp $
+ * @version $Id: UrlEncoded.java,v 1.24 2005/12/21 23:14:38 gregwilkins Exp $
  * @author Greg Wilkins (gregw)
  */
 public class UrlEncoded extends MultiMap
@@ -280,6 +280,8 @@ public class UrlEncoded extends MultiMap
                           data[ox++] = (byte)' ';
                           break;
                       case '%':
+                          int i0 = (14<<4)+1;
+                          byte b0 = (byte)i0;
                           data[ox++] = (byte)
                               ((TypeUtil.convertHexDigit(data[ix++]) << 4)+
                                TypeUtil.convertHexDigit(data[ix++]));
@@ -333,30 +335,22 @@ public class UrlEncoded extends MultiMap
             charset=StringUtil.__ISO_8859_1;
         byte[] bytes=null;
         int n=0;
-        StringBuffer buf=null;
         
         for (int i=0;i<length;i++)
         {
             char c = encoded.charAt(offset+i);
             if (c<0||c>0xff)
-                throw new IllegalArgumentException("Not decoded");
+                throw new IllegalArgumentException("Not encoded");
             
             if (c=='+')
             {
-                if (buf==null)
+                if (bytes==null)
                 {
-                    buf=new StringBuffer(length);
-                    for (int j=0;j<i;j++)
-                        buf.append(encoded.charAt(offset+j));
+                    bytes=new byte[length*2];
+                    encoded.getBytes(offset, offset+i, bytes, 0);
+                    n=i;
                 }
-                if (n>0)
-                {
-                    try {buf.append(new String(bytes,0,n,charset));}
-                    catch(UnsupportedEncodingException e)
-                    {buf.append(new String(bytes,0,n));}
-                    n=0;
-                }        
-                buf.append(' ');
+                bytes[n++] = (byte) ' ';
             }
             else if (c=='%' && (i+2)<length)
             {
@@ -375,46 +369,36 @@ public class UrlEncoded extends MultiMap
                     b=(byte)(b*16+10+cn-'A');
                 else
                     b=(byte)(b*16+cn-'0');
-                
-                if (buf==null)
+
+                if (bytes==null)
                 {
-                    buf=new StringBuffer(length);
-                    for (int j=0;j<i;j++)
-                        buf.append(encoded.charAt(offset+j));
+                    bytes=new byte[length*2];
+                    encoded.getBytes(offset, offset+i, bytes, 0);
+                    n=i;
                 }
                 i+=2;
-                if (bytes==null)
-                    bytes=new byte[length];
                 bytes[n++]=b;
             }
-            else if (buf!=null)
-            {
-                if (n>0)
-                {
-                    try {buf.append(new String(bytes,0,n,charset));}
-                    catch(UnsupportedEncodingException e)
-                    {buf.append(new String(bytes,0,n));}
-                    n=0;
-                }                
-                buf.append(c);
-            }
+            else if (n>0)
+                bytes[n++] = (byte) c;
         }
 
-        if (buf==null)
+        if (bytes==null)
         {
             if (offset==0 && encoded.length()==length)
                 return encoded;
             return encoded.substring(offset,offset+length);
         }
-
-        if (n>0)
+        
+        try
         {
-            try {buf.append(new String(bytes,0,n,charset));}
-            catch(UnsupportedEncodingException e)
-            {buf.append(new String(bytes,0,n));}
+            return new String(bytes,0,n,charset);
         }
-
-        return buf.toString();
+        catch (UnsupportedEncodingException e)
+        {
+            return new String(bytes,0,n);
+        }
+        
     }
     
     /* ------------------------------------------------------------ */
@@ -505,6 +489,6 @@ public class UrlEncoded extends MultiMap
      */
     public Object clone()
     {
-        return new UrlEncoded(this);
+	return super.clone();
     }
 }
